@@ -16,6 +16,8 @@ $Id: undi.c,v 1.8 2003/10/25 13:54:53 mcb30 Exp $
  * your option) any later version.
  */
 
+#ifdef PCBIOS
+
 /* to get some global routines like printf */
 #include "etherboot.h"
 /* to get the interface to the body of the program */
@@ -1151,3 +1153,5 @@ static struct pci_driver undi_driver __pci_driver = {
  	.id_count = sizeof(undi_nics)/sizeof(undi_nics[0]),
 	.class    = PCI_CLASS_NETWORK_ETHERNET,
 };
+
+#endif /* PCBIOS */
