@@ -36,6 +36,7 @@
 *
 *    v1.0	11-26-2003	timlegge	Initial port of Linux driver
 *    v1.5	01-17-2004	timlegge	Initial driver output cleanup
+*    v1.6	03-27-2004	timlegge	Additional Cleanup
 *    
 *    Indent Options: indent -kr -i8
 ***************************************************************************/
@@ -48,8 +49,8 @@
 #include "pci.h"
 #include "timer.h"
 
-#define drv_version "v1.5"
-#define drv_date "01-17-2004"
+#define drv_version "v1.6"
+#define drv_date "03-27-2004"
 
 typedef unsigned char u8;
 typedef signed char s8;
@@ -58,15 +59,30 @@ typedef signed short s16;
 typedef unsigned int u32;
 typedef signed int s32;
 
+#define HZ 1000
+
 static u32 ioaddr;
 
+#ifdef EDEBUG
+#define dprintf(x) printf x
+#else
+#define dprintf(x)
+#endif
+
 /* Condensed operations for readability. */
 #define virt_to_le32desc(addr)  cpu_to_le32(virt_to_bus(addr))
 #define le32desc_to_virt(addr)  bus_to_virt(le32_to_cpu(addr))
 
-/* media options */
-#define MAX_UNITS 8
-static int media[MAX_UNITS] = { -1, -1, -1, -1, -1, -1, -1, -1 };
+#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
+
+/* media options 
+        _10_Half = 0x01,
+        _10_Full = 0x02,
+        _100_Half = 0x04,
+        _100_Full = 0x08,
+        _1000_Full = 0x10,
+*/
+static int media = -1;
 
 /* Maximum events (Rx packets, etc.) to handle at each interrupt. */
 static int max_interrupt_work = 20;
@@ -208,16 +224,16 @@ enum RTL8169_register_content {
 	PHY_Restart_Auto_Nego = 0x0200,
 	PHY_Enable_Auto_Nego = 0x1000,
 
-	//PHY_STAT_REG = 1;
+	/* PHY_STAT_REG = 1; */
 	PHY_Auto_Neco_Comp = 0x0020,
 
-	//PHY_AUTO_NEGO_REG = 4;
+	/* PHY_AUTO_NEGO_REG = 4; */
 	PHY_Cap_10_Half = 0x0020,
 	PHY_Cap_10_Full = 0x0040,
 	PHY_Cap_100_Half = 0x0080,
 	PHY_Cap_100_Full = 0x0100,
 
-	//PHY_1000_CTRL_REG = 9;
+	/* PHY_1000_CTRL_REG = 9; */
 	PHY_Cap_1000_Full = 0x0200,
 
 	PHY_Cap_Null = 0x0,
@@ -262,9 +278,13 @@ struct RxDesc {
 	u32 buf_Haddr;
 };
 
+/* The descriptors for this card are required to be aligned on
+256 byte boundaries.  As the align attribute does not do more than
+16 bytes of alignment it requires some extra steps.  Add 256 to the 
+size of the array and the init_ring adjusts the alignment */
+
 /* Define the TX Descriptor */
 static u8 tx_ring[NUM_TX_DESC * sizeof(struct TxDesc) + 256];
-//	__attribute__ ((aligned(256)));
 
 /* Create a static buffer of size RX_BUF_SZ for each
 TX Descriptor.  All descriptors point to a
@@ -273,7 +293,6 @@ static unsigned char txb[NUM_TX_DESC * RX_BUF_SIZE];
 
 /* Define the RX Descriptor */
 static u8 rx_ring[NUM_RX_DESC * sizeof(struct TxDesc) + 256];
-  //  __attribute__ ((aligned(256)));
 
 /* Create a static buffer of size RX_BUF_SZ for each
 RX Descriptor   All descriptors point to a
@@ -285,12 +304,10 @@ struct rtl8169_private {
 	int chipset;
 	unsigned long cur_rx;	/* Index into the Rx descriptor buffer of next Rx pkt. */
 	unsigned long cur_tx;	/* Index into the Tx descriptor buffer of next Rx pkt. */
-	unsigned long dirty_tx;
 	unsigned char *TxDescArrays;	/* Index of Tx Descriptor buffer */
 	unsigned char *RxDescArrays;	/* Index of Rx Descriptor buffer */
 	struct TxDesc *TxDescArray;	/* Index of 256-alignment Tx Descriptor buffer */
 	struct RxDesc *RxDescArray;	/* Index of 256-alignment Rx Descriptor buffer */
-	unsigned char *RxBufferRings;	/* Index of Rx Buffer  */
 	unsigned char *RxBufferRing[NUM_RX_DESC];	/* Index of Rx Buffer array */
 	unsigned char *Tx_skbuff[NUM_TX_DESC];
 } tpx;
@@ -311,7 +328,7 @@ void mdio_write(int RegAddr, int value)
 	udelay(1000);
 
 	for (i = 2000; i > 0; i--) {
-		// Check if the RTL8169 has completed writing to the specified MII register
+		/* Check if the RTL8169 has completed writing to the specified MII register */
 		if (!(RTL_R32(PHYAR) & 0x80000000)) {
 			break;
 		} else {
@@ -328,7 +345,7 @@ int mdio_read(int RegAddr)
 	udelay(1000);
 
 	for (i = 2000; i > 0; i--) {
-		// Check if the RTL8169 has completed retrieving data from the specified MII register
+		/* Check if the RTL8169 has completed retrieving data from the specified MII register */
 		if (RTL_R32(PHYAR) & 0x80000000) {
 			value = (int) (RTL_R32(PHYAR) & 0xFFFF);
 			break;
@@ -339,42 +356,40 @@ int mdio_read(int RegAddr)
 	return value;
 }
 
-#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
-
 static int rtl8169_init_board(struct pci_device *pdev)
 {
-	int i, pm_cap;
+	int i;
 	unsigned long rtreg_base, rtreg_len;
 	u32 tmp;
 
 	rtreg_base = pci_bar_start(pdev, PCI_BASE_ADDRESS_1);
 	rtreg_len = pci_bar_size(pdev, PCI_BASE_ADDRESS_1);
 
-	// check for weird/broken PCI region reporting
+	/* check for weird/broken PCI region reporting */
 	if (rtreg_len < RTL_MIN_IO_SIZE) {
 		printf("Invalid PCI region size(s), aborting\n");
 	}
 
 	adjust_pci_device(pdev);
-	pm_cap = pci_find_capability(pdev, PCI_CAP_ID_PM);
+/*      pm_cap = pci_find_capability(pdev, PCI_CAP_ID_PM); */
 
-	// ioremap MMIO region 
+	/* ioremap MMIO region */
 	ioaddr = (unsigned long) ioremap(rtreg_base, rtreg_len);
 	if (ioaddr == 0)
-		printf("Error cannot remap MMIO, aborting\n");
+		return 0;
 
 	tpc->mmio_addr = &ioaddr;
-	// Soft reset the chip. 
+	/* Soft reset the chip. */
 	RTL_W8(ChipCmd, CmdReset);
 
-	// Check that the chip has finished the reset.
+	/* Check that the chip has finished the reset. */
 	for (i = 1000; i > 0; i--)
 		if ((RTL_R8(ChipCmd) & CmdReset) == 0)
 			break;
 		else
 			udelay(10);
 
-	// identify chip attached to board
+	/* identify chip attached to board */
 	tmp = RTL_R32(TxConfig);
 	tmp = ((tmp & 0x7c000000) + ((tmp & 0x00800000) << 2)) >> 24;
 
@@ -383,12 +398,12 @@ static int rtl8169_init_board(struct pci_device *pdev)
 			tpc->chipset = i;
 			goto match;
 		}
-	//if unknown chip, assume array element #0, original RTL-8169 in this case
-	printf("PCI device %s: unknown chip version, assuming RTL-8169\n");
-	printf("PCI device: TxConfig = 0x%hX\n",
-	       (unsigned long) RTL_R32(TxConfig));
+	/* if unknown chip, assume array element #0, original RTL-8169 in this case */
+	dprintf(("PCI device: unknown chip version, assuming RTL-8169\n"));
+	dprintf(("PCI device: TxConfig = 0x%hX\n",
+		 (unsigned long) RTL_R32(TxConfig)));
 	tpc->chipset = 0;
-
+	return 1;
       match:
 	return 0;
 
@@ -420,24 +435,26 @@ static int r8169_poll(struct nic *nic)
 			    virt_to_bus(tpc->RxBufferRing[cur_rx]);
 		} else
 			printf("Error Rx");
+		/* FIXME: shouldn't I reset the status on an error */
 		cur_rx = (cur_rx + 1) % NUM_RX_DESC;
 		tpc->cur_rx = cur_rx;
 		return 1;
 
 	}
 	tpc->cur_rx = cur_rx;
+	/* FIXME: There is no reason to do this as cur_rx did not change */
+
 	return (0);		/* initially as this is called to flush the input */
 
 }
 
-#define HZ 1000
 /**************************************************************************
 TRANSMIT - Transmit a frame
 ***************************************************************************/
 static void r8169_transmit(struct nic *nic, const char *d,	/* Destination */
-			    unsigned int t,	/* Type */
-			    unsigned int s,	/* size */
-			    const char *p)
+			   unsigned int t,	/* Type */
+			   unsigned int s,	/* size */
+			   const char *p)
 {				/* Packet */
 	/* send the packet to destination */
 
@@ -461,13 +478,13 @@ static void r8169_transmit(struct nic *nic, const char *d,	/* Destination */
 	tpc->TxDescArray[entry].buf_addr = virt_to_bus(ptxb);
 	if (entry != (NUM_TX_DESC - 1))
 		tpc->TxDescArray[entry].status =
-		    (OWNbit | FSbit | LSbit) | ((s > ETH_ZLEN) ?
-						s : ETH_ZLEN);
+		    (OWNbit | FSbit | LSbit) | ((s > ETH_ZLEN) ? s :
+						ETH_ZLEN);
 	else
 		tpc->TxDescArray[entry].status =
-		    (OWNbit | EORbit | FSbit | LSbit) |
-		    ((s > ETH_ZLEN) ? s : ETH_ZLEN);
-	RTL_W8(TxPoll, 0x40);	//set polling bit
+		    (OWNbit | EORbit | FSbit | LSbit) | ((s > ETH_ZLEN) ? s
+							 : ETH_ZLEN);
+	RTL_W8(TxPoll, 0x40);	/* set polling bit */
 
 	tpc->cur_tx++;
 	to = currticks() + TX_TIMEOUT;
@@ -517,10 +534,10 @@ static void rtl8169_hw_start(struct nic *nic)
 	RTL_W8(ChipCmd, CmdTxEnb | CmdRxEnb);
 	RTL_W8(EarlyTxThres, EarlyTxThld);
 
-	// For gigabit rtl8169
+	/* For gigabit rtl8169 */
 	RTL_W16(RxMaxSize, RxPacketMaxSize);
 
-	// Set Rx Config register
+	/* Set Rx Config register */
 	i = rtl8169_rx_config | (RTL_R32(RxConfig) &
 				 rtl_chip_info[tpc->chipset].RxConfigMask);
 	RTL_W32(RxConfig, i);
@@ -552,7 +569,6 @@ static void rtl8169_init_ring(struct nic *nic __unused)
 
 	tpc->cur_rx = 0;
 	tpc->cur_tx = 0;
-	tpc->dirty_tx = 0;
 	memset(tpc->TxDescArray, 0x0, NUM_TX_DESC * sizeof(struct TxDesc));
 	memset(tpc->RxDescArray, 0x0, NUM_RX_DESC * sizeof(struct RxDesc));
 
@@ -582,17 +598,17 @@ static void r8169_reset(struct nic *nic)
 	u8 diff;
 	u32 TxPhyAddr, RxPhyAddr;
 
-	tpc->TxDescArrays = tx_ring; 
+	tpc->TxDescArrays = tx_ring;
 	if (tpc->TxDescArrays == 0)
 		printf("Allot Error");
-	// Tx Desscriptor needs 256 bytes alignment;
+	/* Tx Desscriptor needs 256 bytes alignment; */
 	TxPhyAddr = virt_to_bus(tpc->TxDescArrays);
 	diff = 256 - (TxPhyAddr - ((TxPhyAddr >> 8) << 8));
 	TxPhyAddr += diff;
 	tpc->TxDescArray = (struct TxDesc *) (tpc->TxDescArrays + diff);
 
 	tpc->RxDescArrays = rx_ring;
-	// Rx Desscriptor needs 256 bytes alignment;
+	/* Rx Desscriptor needs 256 bytes alignment; */
 	RxPhyAddr = virt_to_bus(tpc->RxDescArrays);
 	diff = 256 - (RxPhyAddr - ((RxPhyAddr >> 8) << 8));
 	RxPhyAddr += diff;
@@ -616,8 +632,6 @@ static void r8169_reset(struct nic *nic)
 	txb[3] = nic->node_addr[3];
 	txb[4] = nic->node_addr[4];
 	txb[5] = nic->node_addr[5];
-
-
 }
 
 /**************************************************************************
@@ -657,12 +671,8 @@ static int r8169_probe(struct dev *dev, struct pci_device *pci)
 	int i, rc;
 	int option = -1, Cap10_100 = 0, Cap1000 = 0;
 
-	/* BASE is used throughout to address the card */
-	printf("\n");
-	printf
-	    ("r8169.c: %s, %s\n", drv_version, drv_date);
-	printf("%s: Probing for Vendor=%hX   Device=%hX, %s\n", pci->name,
-	       pci->vendor, pci->dev_id);
+	printf("r8169.c: Found %s, Vendor=%hX Device=%hX\n",
+	       pci->name, pci->vendor, pci->dev_id);
 
 	board_idx++;
 
@@ -671,27 +681,26 @@ static int r8169_probe(struct dev *dev, struct pci_device *pci)
 	/* point to private storage */
 	tpc = &tpx;
 
-	rc = rtl8169_init_board(pci);
-	if (rc)
-		return rc;
+	rc = rtl8169_init_board(pci);	/* Return code is meaningless */
 
-	// Get MAC address.  FIXME: read EEPROM
+	/* Get MAC address.  FIXME: read EEPROM */
 	for (i = 0; i < MAC_ADDR_LEN; i++)
 		nic->node_addr[i] = RTL_R8(MAC0 + i);
 
-	printf("%s: Identified chip type is '%s'.\n", pci->name,
-	       rtl_chip_info[tpc->chipset].name);
+	dprintf(("%s: Identified chip type is '%s'.\n", pci->name,
+		 rtl_chip_info[tpc->chipset].name));
 	/* Print out some hardware info */
-	printf("%s%s: %! at ioaddr %hX\n", pci->name, nic->node_addr, ioaddr);
+	printf("%s: %! at ioaddr %hX, ", pci->name, nic->node_addr,
+	       ioaddr);
 
-	// if TBI is not endbled
+	/* if TBI is not endbled */
 	if (!(RTL_R8(PHYstatus) & TBI_Enable)) {
 		int val = mdio_read(PHY_AUTO_NEGO_REG);
 
-		option = (board_idx >= MAX_UNITS) ? 0 : media[board_idx];
-		// Force RTL8169 in 10/100/1000 Full/Half mode.
+		option = media;
+		/* Force RTL8169 in 10/100/1000 Full/Half mode. */
 		if (option > 0) {
-			printf("%s: Force-mode Enabled.\n", pci->name);
+			printf(" Force-mode Enabled.\n");
 			Cap10_100 = 0, Cap1000 = 0;
 			switch (option) {
 			case _10_Half:
@@ -717,42 +726,42 @@ static int r8169_probe(struct dev *dev, struct pci_device *pci)
 			default:
 				break;
 			}
-			mdio_write(PHY_AUTO_NEGO_REG, Cap10_100 | (val & 0x1F));	//leave PHY_AUTO_NEGO_REG bit4:0 unchanged
+			/* leave PHY_AUTO_NEGO_REG bit4:0 unchanged */
+			mdio_write(PHY_AUTO_NEGO_REG,
+				   Cap10_100 | (val & 0x1F));
 			mdio_write(PHY_1000_CTRL_REG, Cap1000);
 		} else {
-			printf("%s: Auto-negotiation Enabled.\n",
-			       pci->name);
+			dprintf(("Auto-negotiation Enabled.\n",
+				 pci->name));
 
-			// enable 10/100 Full/Half Mode, leave PHY_AUTO_NEGO_REG bit4:0 unchanged
+			/* enable 10/100 Full/Half Mode, leave PHY_AUTO_NEGO_REG bit4:0 unchanged */
 			mdio_write(PHY_AUTO_NEGO_REG,
 				   PHY_Cap_10_Half | PHY_Cap_10_Full |
 				   PHY_Cap_100_Half | PHY_Cap_100_Full |
 				   (val & 0x1F));
 
-			// enable 1000 Full Mode
+			/* enable 1000 Full Mode */
 			mdio_write(PHY_1000_CTRL_REG, PHY_Cap_1000_Full);
 
 		}
 
-		// Enable auto-negotiation and restart auto-nigotiation
+		/* Enable auto-negotiation and restart auto-nigotiation */
 		mdio_write(PHY_CTRL_REG,
 			   PHY_Enable_Auto_Nego | PHY_Restart_Auto_Nego);
 		udelay(100);
 
-		// wait for auto-negotiation process
+		/* wait for auto-negotiation process */
 		for (i = 10000; i > 0; i--) {
-			//check if auto-negotiation complete
+			/* Check if auto-negotiation complete */
 			if (mdio_read(PHY_STAT_REG) & PHY_Auto_Neco_Comp) {
 				udelay(100);
 				option = RTL_R8(PHYstatus);
 				if (option & _1000bpsF) {
 					printf
-					    ("%s: 1000Mbps Full-duplex operation.\n",
-					     pci->name);
+					    ("1000Mbps Full-duplex operation.\n");
 				} else {
 					printf
-					    ("%s: %sMbps %s-duplex operation.\n",
-					     pci->name,
+					    ("%sMbps %s-duplex operation.\n",
 					     (option & _100bps) ? "100" :
 					     "10",
 					     (option & FullDup) ? "Full" :
@@ -762,7 +771,7 @@ static int r8169_probe(struct dev *dev, struct pci_device *pci)
 			} else {
 				udelay(100);
 			}
-		}		// end for-loop to wait for auto-negotiation process
+		}		/* end for-loop to wait for auto-negotiation process */
 
 	} else {
 		udelay(100);
